@@ -1,7 +1,7 @@
 use super::logical::LogicalExpr;
 use crate::data::{
+    list::{List, Values},
     scalar::Scalar,
-    vector::{F32SparseVector, F32Vector, SparseVector, Vector},
 };
 use pyo3::{
     exceptions::PyTypeError,
@@ -9,10 +9,6 @@ use pyo3::{
     types::{PyBool, PyFloat, PyInt, PyString},
 };
 
-#[pyclass]
-#[derive(Debug, Clone)]
-pub struct Null;
-
 #[derive(Debug, Clone)]
 pub enum FlexibleExpr {
     String(String),
@@ -130,35 +126,71 @@ impl Into<LogicalExpr> for Stringy {
     }
 }
 
-#[derive(Debug, Clone)]
-pub enum Vectorish {
-    DenseU8(Vec<u8>),
-    DenseF32(Vec<f32>),
-    SparseU8(Vec<u32>, Vec<u8>),
-    SparseF32(Vec<u32>, Vec<f32>),
+#[derive(Debug, Clone, FromPyObject)]
+pub enum StringyWithList {
+    Stringy(Stringy),
+    List(Vec<String>),
 }
 
-impl<'py> FromPyObject<'py> for Vectorish {
-    fn extract_bound(obj: &Bound<'py, PyAny>) -> PyResult<Self> {
-        if let Ok(v) = obj.downcast::<Vector>() {
-            match v.get().clone() {
-                Vector::F32(values) => Ok(Vectorish::DenseF32(values)),
-                Vector::U8(values) => Ok(Vectorish::DenseU8(values)),
-            }
-        } else if let Ok(v) = obj.downcast::<SparseVector>() {
-            match v.get().clone() {
-                SparseVector::F32 { indices, values } => Ok(Vectorish::SparseF32(indices, values)),
-                SparseVector::U8 { indices, values } => Ok(Vectorish::SparseU8(indices, values)),
-            }
-        } else if let Ok(v) = F32Vector::extract_bound(obj) {
-            Ok(Vectorish::DenseF32(v.values))
-        } else if let Ok(v) = F32SparseVector::extract_bound(obj) {
-            Ok(Vectorish::SparseF32(v.indices, v.values))
-        } else {
-            Err(PyTypeError::new_err(format!(
-                "Can't convert from {:?} to Vectorish",
-                obj.get_type().name()
-            )))
+impl Into<LogicalExpr> for StringyWithList {
+    fn into(self) -> LogicalExpr {
+        match self {
+            StringyWithList::Stringy(s) => s.into(),
+            StringyWithList::List(values) => LogicalExpr::Literal {
+                value: Scalar::List(List {
+                    values: Values::String(values),
+                }),
+            },
+        }
+    }
+}
+
+#[derive(Debug, Clone, FromPyObject)]
+pub enum Iterable {
+    #[pyo3(transparent)]
+    String(String),
+
+    #[pyo3(transparent)]
+    List(List),
+
+    #[pyo3(transparent)]
+    StringList(Vec<String>),
+
+    #[pyo3(transparent)]
+    IntList(Vec<i64>),
+
+    #[pyo3(transparent)]
+    FloatList(Vec<f32>),
+
+    #[pyo3(transparent)]
+    Expr(LogicalExpr),
+}
+
+impl Into<LogicalExpr> for Iterable {
+    fn into(self) -> LogicalExpr {
+        match self {
+            Iterable::String(s) => LogicalExpr::Literal {
+                value: Scalar::String(s),
+            },
+            Iterable::List(l) => LogicalExpr::Literal {
+                value: Scalar::List(l),
+            },
+            Iterable::StringList(l) => LogicalExpr::Literal {
+                value: Scalar::List(List {
+                    values: Values::String(l),
+                }),
+            },
+            Iterable::IntList(l) => LogicalExpr::Literal {
+                value: Scalar::List(List {
+                    values: Values::I64(l),
+                }),
+            },
+            Iterable::FloatList(l) => LogicalExpr::Literal {
+                value: Scalar::List(List {
+                    values: Values::F32(l),
+                }),
+            },
+            Iterable::Expr(e) => e,
         }
     }
 }
