@@ -1,15 +1,19 @@
 use crate::data::scalar::Scalar;
 use crate::expr::flexible::FlexibleExpr;
+use crate::expr::flexible::{Boolish, Iterable, Numeric, Stringy, StringyWithList};
 use pyo3::prelude::*;
 
-use super::flexible::{Boolish, Numeric, Stringy};
-
 #[derive(Debug, Clone, Copy, PartialEq)]
 #[pyclass(eq, eq_int)]
 pub enum UnaryOperator {
     Not,
     IsNull,
     IsNotNull,
+    Abs,
+    Ln,
+    Exp,
+    Sqrt,
+    Square,
 }
 
 impl From<UnaryOperator> for topk_rs::proto::v1::data::logical_expr::unary_op::Op {
@@ -20,6 +24,11 @@ impl From<UnaryOperator> for topk_rs::proto::v1::data::logical_expr::unary_op::O
             UnaryOperator::IsNotNull => {
                 topk_rs::proto::v1::data::logical_expr::unary_op::Op::IsNotNull
             }
+            UnaryOperator::Abs => topk_rs::proto::v1::data::logical_expr::unary_op::Op::Abs,
+            UnaryOperator::Ln => topk_rs::proto::v1::data::logical_expr::unary_op::Op::Ln,
+            UnaryOperator::Exp => topk_rs::proto::v1::data::logical_expr::unary_op::Op::Exp,
+            UnaryOperator::Sqrt => topk_rs::proto::v1::data::logical_expr::unary_op::Op::Sqrt,
+            UnaryOperator::Square => topk_rs::proto::v1::data::logical_expr::unary_op::Op::Square,
         }
     }
 }
@@ -27,11 +36,9 @@ impl From<UnaryOperator> for topk_rs::proto::v1::data::logical_expr::unary_op::O
 #[derive(Debug, Clone, Copy, PartialEq)]
 #[pyclass(eq, eq_int)]
 pub enum BinaryOperator {
-    // Logical ops
     And,
     Or,
     Xor,
-    // Comparison ops
     Eq,
     NotEq,
     Lt,
@@ -40,16 +47,17 @@ pub enum BinaryOperator {
     GtEq,
     StartsWith,
     Contains,
-    // Arithmetic ops
+    In,
     Add,
     Sub,
     Mul,
     Div,
     Rem,
-    // Other
     MatchAll,
     MatchAny,
     Coalesce,
+    Min,
+    Max,
 }
 
 impl From<BinaryOperator> for topk_rs::proto::v1::data::logical_expr::binary_op::Op {
@@ -69,6 +77,7 @@ impl From<BinaryOperator> for topk_rs::proto::v1::data::logical_expr::binary_op:
             BinaryOperator::Contains => {
                 topk_rs::proto::v1::data::logical_expr::binary_op::Op::Contains
             }
+            BinaryOperator::In => topk_rs::proto::v1::data::logical_expr::binary_op::Op::In,
             BinaryOperator::MatchAll => {
                 topk_rs::proto::v1::data::logical_expr::binary_op::Op::MatchAll
             }
@@ -84,6 +93,8 @@ impl From<BinaryOperator> for topk_rs::proto::v1::data::logical_expr::binary_op:
             }
             BinaryOperator::Rem => unimplemented!("`rem` operator is not supported"),
             BinaryOperator::Xor => unimplemented!("`xor` operator is not supported"),
+            BinaryOperator::Min => topk_rs::proto::v1::data::logical_expr::binary_op::Op::Min,
+            BinaryOperator::Max => topk_rs::proto::v1::data::logical_expr::binary_op::Op::Max,
         }
     }
 }
@@ -104,6 +115,22 @@ impl From<TernaryOperator> for topk_rs::proto::v1::data::logical_expr::ternary_o
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq)]
+#[pyclass(eq, eq_int)]
+pub enum NaryOperator {
+    All,
+    Any,
+}
+
+impl From<NaryOperator> for topk_rs::proto::v1::data::logical_expr::nary_op::Op {
+    fn from(op: NaryOperator) -> Self {
+        match op {
+            NaryOperator::All => topk_rs::proto::v1::data::logical_expr::nary_op::Op::All,
+            NaryOperator::Any => topk_rs::proto::v1::data::logical_expr::nary_op::Op::Any,
+        }
+    }
+}
+
 #[pyclass]
 #[derive(Clone)]
 pub enum LogicalExpr {
@@ -128,6 +155,10 @@ pub enum LogicalExpr {
         y: Py<LogicalExpr>,
         z: Py<LogicalExpr>,
     },
+    Nary {
+        op: NaryOperator,
+        exprs: Vec<Py<LogicalExpr>>,
+    },
 }
 
 impl std::fmt::Debug for LogicalExpr {
@@ -157,6 +188,9 @@ impl std::fmt::Debug for LogicalExpr {
                     z.get()
                 )
             }
+            Self::Nary { op, exprs } => {
+                write!(f, "Nary(op={:?}, exprs={:?})", op, exprs)
+            }
         }
     }
 }
@@ -207,6 +241,23 @@ impl PartialEq for LogicalExpr {
                     && l_y.get() == r_y.get()
                     && l_z.get() == r_z.get()
             }
+            (
+                LogicalExpr::Nary {
+                    op: l_op,
+                    exprs: l_exprs,
+                },
+                LogicalExpr::Nary {
+                    op: r_op,
+                    exprs: r_exprs,
+                },
+            ) => {
+                l_op == r_op
+                    && l_exprs.len() == r_exprs.len()
+                    && l_exprs
+                        .iter()
+                        .zip(r_exprs.iter())
+                        .all(|(l, r)| l.get() == r.get())
+            }
             _ => false,
         }
     }
@@ -222,8 +273,6 @@ impl LogicalExpr {
         self == other
     }
 
-    // Unary operators
-
     fn is_null(&self, py: Python<'_>) -> PyResult<Self> {
         Ok(Self::Unary {
             op: UnaryOperator::IsNull,
@@ -238,6 +287,45 @@ impl LogicalExpr {
         })
     }
 
+    fn abs(&self, py: Python<'_>) -> PyResult<Self> {
+        Ok(Self::Unary {
+            op: UnaryOperator::Abs,
+            expr: Py::new(py, self.clone())?,
+        })
+    }
+
+    fn __abs__(&self, py: Python<'_>) -> PyResult<Self> {
+        self.abs(py)
+    }
+
+    fn ln(&self, py: Python<'_>) -> PyResult<Self> {
+        Ok(Self::Unary {
+            op: UnaryOperator::Ln,
+            expr: Py::new(py, self.clone())?,
+        })
+    }
+
+    fn exp(&self, py: Python<'_>) -> PyResult<Self> {
+        Ok(Self::Unary {
+            op: UnaryOperator::Exp,
+            expr: Py::new(py, self.clone())?,
+        })
+    }
+
+    fn sqrt(&self, py: Python<'_>) -> PyResult<Self> {
+        Ok(Self::Unary {
+            op: UnaryOperator::Sqrt,
+            expr: Py::new(py, self.clone())?,
+        })
+    }
+
+    fn square(&self, py: Python<'_>) -> PyResult<Self> {
+        Ok(Self::Unary {
+            op: UnaryOperator::Square,
+            expr: Py::new(py, self.clone())?,
+        })
+    }
+
     fn eq(&self, py: Python<'_>, other: FlexibleExpr) -> PyResult<Self> {
         let expr: LogicalExpr = other.into();
 
@@ -480,7 +568,7 @@ impl LogicalExpr {
         })
     }
 
-    fn contains(&self, py: Python<'_>, other: Stringy) -> PyResult<Self> {
+    fn contains(&self, py: Python<'_>, other: FlexibleExpr) -> PyResult<Self> {
         Ok(Self::Binary {
             left: Py::new(py, self.clone())?,
             op: BinaryOperator::Contains,
@@ -488,7 +576,15 @@ impl LogicalExpr {
         })
     }
 
-    fn match_all(&self, py: Python<'_>, other: Stringy) -> PyResult<Self> {
+    fn in_(&self, py: Python<'_>, other: Iterable) -> PyResult<Self> {
+        Ok(Self::Binary {
+            left: Py::new(py, self.clone())?,
+            op: BinaryOperator::In,
+            right: Py::new(py, Into::<LogicalExpr>::into(other))?,
+        })
+    }
+
+    fn match_all(&self, py: Python<'_>, other: StringyWithList) -> PyResult<Self> {
         Ok(Self::Binary {
             left: Py::new(py, self.clone())?,
             op: BinaryOperator::MatchAll,
@@ -496,7 +592,7 @@ impl LogicalExpr {
         })
     }
 
-    fn match_any(&self, py: Python<'_>, other: Stringy) -> PyResult<Self> {
+    fn match_any(&self, py: Python<'_>, other: StringyWithList) -> PyResult<Self> {
         Ok(Self::Binary {
             left: Py::new(py, self.clone())?,
             op: BinaryOperator::MatchAny,
@@ -532,6 +628,22 @@ impl LogicalExpr {
         let choose_numeric = Numeric::Expr(choose_expr);
         self.mul(py, choose_numeric)
     }
+
+    fn min(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+        Ok(Self::Binary {
+            left: Py::new(py, self.clone())?,
+            op: BinaryOperator::Min,
+            right: Py::new(py, Into::<LogicalExpr>::into(other))?,
+        })
+    }
+
+    fn max(&self, py: Python<'_>, other: Numeric) -> PyResult<Self> {
+        Ok(Self::Binary {
+            left: Py::new(py, self.clone())?,
+            op: BinaryOperator::Max,
+            right: Py::new(py, Into::<LogicalExpr>::into(other))?,
+        })
+    }
 }
 
 impl From<LogicalExpr> for topk_rs::proto::v1::data::LogicalExpr {
@@ -555,6 +667,10 @@ impl From<LogicalExpr> for topk_rs::proto::v1::data::LogicalExpr {
                 y.get().clone(),
                 z.get().clone(),
             ),
+            LogicalExpr::Nary { op, exprs } => topk_rs::proto::v1::data::LogicalExpr::nary(
+                op,
+                exprs.into_iter().map(|e| e.get().clone()),
+            ),
         }
     }
 }
