@@ -1,12 +1,13 @@
 use crate::data::scalar::Scalar;
-use crate::data::vector::{SparseVector, Vector};
+use crate::data::value::Value;
 use crate::expr::filter::FilterExprUnion;
-use crate::expr::flexible::Vectorish;
-use crate::expr::function::{FunctionExpr, QueryVector};
-use crate::expr::logical::{LogicalExpr, UnaryOperator};
+use crate::expr::flexible::Numeric;
+use crate::expr::function::FunctionExpr;
+use crate::expr::logical::{BinaryOperator, LogicalExpr, NaryOperator, UnaryOperator};
 use crate::expr::select::SelectExprUnion;
 use crate::expr::text::{Term, TextExpr};
 use crate::module;
+use pyo3::exceptions::PyValueError;
 use pyo3::prelude::*;
 use std::collections::HashMap;
 
@@ -32,6 +33,11 @@ pub fn pymodule(m: &Bound<'_, PyModule>) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(literal))?;
     m.add_wrapped(wrap_pyfunction!(r#match))?;
     m.add_wrapped(wrap_pyfunction!(not_))?;
+    m.add_wrapped(wrap_pyfunction!(min))?;
+    m.add_wrapped(wrap_pyfunction!(max))?;
+    m.add_wrapped(wrap_pyfunction!(abs))?;
+    m.add_wrapped(wrap_pyfunction!(all))?;
+    m.add_wrapped(wrap_pyfunction!(any))?;
     Ok(())
 }
 
@@ -81,6 +87,54 @@ pub fn not_(py: Python<'_>, expr: LogicalExpr) -> PyResult<LogicalExpr> {
     })
 }
 
+#[pyfunction]
+pub fn all(py: Python<'_>, exprs: Vec<LogicalExpr>) -> PyResult<LogicalExpr> {
+    Ok(LogicalExpr::Nary {
+        op: NaryOperator::All,
+        exprs: exprs
+            .into_iter()
+            .map(|e| Py::new(py, e))
+            .collect::<Result<Vec<Py<LogicalExpr>>, PyErr>>()?,
+    })
+}
+
+#[pyfunction]
+pub fn any(py: Python<'_>, exprs: Vec<LogicalExpr>) -> PyResult<LogicalExpr> {
+    Ok(LogicalExpr::Nary {
+        op: NaryOperator::Any,
+        exprs: exprs
+            .into_iter()
+            .map(|e| Py::new(py, e))
+            .collect::<Result<Vec<Py<LogicalExpr>>, PyErr>>()?,
+    })
+}
+
+#[pyfunction]
+pub fn min(py: Python<'_>, left: Numeric, right: Numeric) -> PyResult<LogicalExpr> {
+    Ok(LogicalExpr::Binary {
+        left: Py::new(py, Into::<LogicalExpr>::into(left))?,
+        op: BinaryOperator::Min,
+        right: Py::new(py, Into::<LogicalExpr>::into(right))?,
+    })
+}
+
+#[pyfunction]
+pub fn max(py: Python<'_>, left: Numeric, right: Numeric) -> PyResult<LogicalExpr> {
+    Ok(LogicalExpr::Binary {
+        left: Py::new(py, Into::<LogicalExpr>::into(left))?,
+        op: BinaryOperator::Max,
+        right: Py::new(py, Into::<LogicalExpr>::into(right))?,
+    })
+}
+
+#[pyfunction]
+pub fn abs(py: Python<'_>, expr: LogicalExpr) -> PyResult<LogicalExpr> {
+    Ok(LogicalExpr::Unary {
+        op: UnaryOperator::Abs,
+        expr: Py::new(py, expr)?,
+    })
+}
+
 #[pymodule]
 #[pyo3(name = "fn")]
 pub fn fn_pymodule(m: &Bound<'_, PyModule>) -> PyResult<()> {
@@ -97,19 +151,22 @@ pub fn bm25_score() -> FunctionExpr {
 }
 
 #[pyfunction]
-pub fn vector_distance(field: String, query: Vectorish) -> FunctionExpr {
-    FunctionExpr::VectorScore {
-        field,
-        query: match query {
-            Vectorish::DenseF32(values) => QueryVector::Dense(Vector::F32(values)),
-            Vectorish::DenseU8(values) => QueryVector::Dense(Vector::U8(values)),
-            Vectorish::SparseF32(indices, values) => {
-                QueryVector::Sparse(SparseVector::F32 { indices, values })
-            }
-            Vectorish::SparseU8(indices, values) => {
-                QueryVector::Sparse(SparseVector::U8 { indices, values })
-            }
-        },
+#[pyo3(signature = (field, query, skip_refine=false))]
+pub fn vector_distance(field: String, query: Value, skip_refine: bool) -> PyResult<FunctionExpr> {
+    match query {
+        Value::List(list) => Ok(FunctionExpr::VectorScore {
+            field,
+            query: Value::List(list),
+            skip_refine,
+        }),
+        Value::SparseVector(vector) => Ok(FunctionExpr::VectorScore {
+            field,
+            query: Value::SparseVector(vector),
+            skip_refine,
+        }),
+        _ => Err(PyValueError::new_err(
+            "Vector query must be a vector or sparse vector",
+        )),
     }
 }
 
