@@ -5,13 +5,13 @@ from topk_sdk.data import bytes
 def test_bytes_from_list():
     result = bytes([0, 1, 255, 128])
     assert result is not None
-    assert str(result) == "Bytes([0, 1, 255, 128])"
+    assert str(result) == str(b'\x00\x01\xff\x80')
 
 
 def test_bytes_from_bytes():
     result = bytes(b"\x00\x01\xff\x80")
     assert result is not None
-    assert str(result) == "Bytes([0, 1, 255, 128])"
+    assert str(result) == str(b'\x00\x01\xff\x80')
 
 
 def test_bytes_invalid_input():
@@ -27,8 +27,8 @@ def test_bytes_empty():
     result2 = bytes(b"")
     assert result1 is not None
     assert result2 is not None
-    assert str(result1) == "Bytes([])"
-    assert str(result2) == "Bytes([])"
+    assert str(result1) == "b''"
+    assert str(result2) == "b''"
 
 
 def test_bytes_with_invalid_list_values():
