@@ -10,6 +10,7 @@ from topk_sdk.schema import (
     u8_sparse_vector,
     u8_vector,
     vector_index,
+    list,
 )
 
 from .. import ProjectContext
@@ -36,6 +37,8 @@ class books:
             "sparse_u8_embedding": u8_sparse_vector().index(
                 vector_index(metric="dot_product")
             ),
+            "tags": list(value_type="text").index(keyword_index()),
+            "codes": list(value_type="text"),
         }
 
         collection = ctx.client.collections().create(
@@ -64,6 +67,9 @@ class books:
                     {0: 1.0, 1: 2.0, 2: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({0: 1, 1: 2, 2: 3}),
+                "tags": ["racism", "injustice", "girl", "father", "lawyer"],
+                "codes": ["ISBN 978-0-06-093546-7", "ISBN 0-06-093546-4", "LCCN 60007854", "UPC 025192354670"],
+                "reprint_years": data.u32_list([1966, 1988, 1999, 2002, 2015]),
             },
             {
                 "_id": "1984",
@@ -78,6 +84,9 @@ class books:
                     {2: 1.0, 3: 2.0, 4: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({2: 1, 3: 2, 4: 3}),
+                "tags": ["dystopia", "surveillance", "totalitarianism", "mind control", "oppression"],
+                "codes": ["1984", "ISBN 978-0-452-28423-4", "ISBN 0-452-28423-6", "OCLC 70775700", "DOI 10.1000/182", "EAN 9780452284234"],
+                "reprint_years": data.u32_list([1950, 1954, 1956, 1961, 1984, 1990, 2003]),
             },
             {
                 "_id": "pride",
@@ -89,6 +98,9 @@ class books:
                     {3: 1.0, 4: 2.0, 5: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({3: 1, 4: 2, 5: 3}),
+                "tags": ["pride", "love", "romance", "class", "marriage", "prejudice"],
+                "codes": ["ISBN 978-0-14-143951-8", "OCLC 934546789"],
+                "reprint_years": data.u32_list([1966, 1972, 1985]),
             },
             {
                 "_id": "gatsby",
@@ -100,6 +112,9 @@ class books:
                     {4: 1.0, 5: 2.0, 6: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({4: 1, 5: 2, 6: 3}),
+                "tags": ["love", "romance", "wealth", "marriage"],
+                "codes": ["ISBN 978-0-7432-7356-5", "ISBN 0-7432-7356-7", "OCLC 60393320", "ASIN B000FC0SIS", "UPC 074327356709", "LCCN 2002114274"],
+                "reprint_years": data.u32_list([1953, 1996, 2004]),
             },
             {
                 "_id": "catcher",
@@ -114,6 +129,9 @@ class books:
                     {5: 1.0, 6: 2.0, 7: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({5: 1, 6: 2, 7: 3}),
+                "tags": ["alienation", "identity", "rebellion", "mid-20th-century", "america"],
+                "codes": ["ISBN 978-0-316-76917-4", "LCCN 51011564", "OCLC 287628"],
+                "reprint_years": data.u32_list([1964, 1979, 1991, 2001, 2010]),
             },
             {
                 "_id": "moby",
@@ -126,6 +144,9 @@ class books:
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({6: 1, 7: 2, 8: 3}),
                 "nullable_importance": 5.0,
+                "tags": ["whale", "obsession", "tragedy", "sailing", "ocean"],
+                "codes": data.string_list([]),
+                "reprint_years": data.u32_list([]),
             },
             {
                 "_id": "hobbit",
@@ -137,6 +158,8 @@ class books:
                     {7: 1.0, 8: 2.0, 9: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({7: 1, 8: 2, 9: 3}),
+                "tags": ["hobbit", "dwarf", "quest", "home", "adventure"],
+                "codes": data.string_list([]),
             },
             {
                 "_id": "harry",
@@ -151,6 +174,9 @@ class books:
                     {8: 1.0, 9: 2.0, 10: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({8: 1, 9: 2, 10: 3}),
+                "tags": ["wizard", "magic", "sorcerer", "school", "witchcraft"],
+                "codes": ["ISBN 978-0-439-70818-0", "UPC 043970818909"],
+                "reprint_years": data.u32_list([1998, 1999, 2001, 2004, 2013, 2020]),
             },
             {
                 "_id": "lotr",
@@ -162,6 +188,8 @@ class books:
                     {9: 1.0, 10: 2.0, 11: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({9: 1, 10: 2, 11: 3}),
+                "tags": ["lord of the rings", "fellowship", "magic", "wizard", "elves"],
+                "codes": ["ISBN 978-0-547-92821-0", "ISBN 0-547-92821-2", "OCLC 434394005", "LCCN 2004558654", "Barcode 0618346252"],
             },
             {
                 "_id": "alchemist",
@@ -173,6 +201,9 @@ class books:
                     {10: 1.0, 11: 2.0, 12: 3.0}
                 ),
                 "sparse_u8_embedding": data.u8_sparse_vector({10: 1, 11: 2, 12: 3}),
+                "tags": ["journey", "destiny", "meaning of life", "alchemy", "soul"],
+                "codes": ["ISBN 978-0-06-231500-7", "ASIN 0062315005", "OCLC 804616251", "DOI 10.1234/alchemist", "EAN 9780062315007", "UPC 006231500719", "LCCN 88675123"],
+                "reprint_years": data.u32_list([1993, 2005, 2014]),
             },
         ]
 
