@@ -1,13 +1,10 @@
 use test_context::test_context;
-use topk_rs::doc;
-use topk_rs::query::field;
-use topk_rs::query::filter;
+use topk_rs::data::literal;
+use topk_rs::query::{field, filter, not, r#match, select};
+use topk_rs::{doc, Error};
 
 mod utils;
-use topk_rs::query::not;
-use topk_rs::query::select;
-use utils::dataset;
-use utils::ProjectTestContext;
+use utils::{dataset, ProjectTestContext};
 
 #[test_context(ProjectTestContext)]
 #[tokio::test]
@@ -300,3 +297,100 @@ async fn test_query_coalesce_non_nullable(ctx: &mut ProjectTestContext) {
         ]
     );
 }
+
+#[test_context(ProjectTestContext)]
+#[tokio::test]
+async fn test_query_abs(ctx: &mut ProjectTestContext) {
+    let collection = dataset::books::setup(ctx).await;
+
+    let result = ctx
+        .client
+        .collection(&collection.name)
+        .query(
+            select([("abs_year", field("published_year").add(-1990).abs())]).topk(
+                field("abs_year"),
+                3,
+                true,
+            ),
+            None,
+            None,
+        )
+        .await
+        .expect("could not query");
+
+    // The 3 books closest to 1990
+    assert_eq!(
+        result,
+        vec![
+            doc!("_id" => "alchemist", "abs_year" => 2_i64),
+            doc!("_id" => "harry", "abs_year" => 7_i64),
+            doc!("_id" => "mockingbird", "abs_year" => 30_i64),
+        ]
+    );
+}
+
+#[test_context(ProjectTestContext)]
+#[tokio::test]
+async fn test_query_topk_min_max(ctx: &mut ProjectTestContext) {
+    let collection = dataset::books::setup(ctx).await;
+
+    let result = ctx
+        .client
+        .collection(&collection.name)
+        .query(
+            select([(
+                "clamped_year",
+                field("published_year").max(1890u32).min(1910u32),
+            )])
+            .filter(r#match(
+                "millionaire love consequences dwarves",
+                Some("summary"),
+                None,
+                false,
+            ))
+            .topk(field("published_year"), 3, true),
+            None,
+            None,
+        )
+        .await
+        .expect("could not query");
+
+    for (i, (id, year)) in [("pride", 1890u32), ("moby", 1890), ("gatsby", 1910)]
+        .iter()
+        .enumerate()
+    {
+        assert_eq!(result[i].id().unwrap(), *id);
+        assert_eq!(
+            result[i].fields.get("clamped_year").unwrap(),
+            &literal(*year)
+        );
+    }
+}
+
+#[test_context(ProjectTestContext)]
+#[tokio::test]
+async fn test_query_logical_deep_recursion_limit(ctx: &mut ProjectTestContext) {
+    let collection = dataset::books::setup(ctx).await;
+
+    // Build a deeply nested OR expression that should trigger recursion limit
+    // Start with a simple logical filter
+    let mut deep_expr = field("published_year").eq(1990u32);
+
+    // Add nested OR expressions to exceed router depth (16) but stay below protobuf decode limit
+    for i in 0..20 {
+        deep_expr = deep_expr.or(field("published_year").eq(1990u32 + i));
+    }
+
+    let err = ctx
+        .client
+        .collection(&collection.name)
+        .query(
+            filter(deep_expr).topk(field("published_year"), 100, true),
+            None,
+            None,
+        )
+        .await
+        .expect_err("should have failed due to recursion limit");
+
+    assert!(matches!(err, Error::InvalidArgument(_) if err.to_string().contains("too deep")));
+}
